<?php

/**
 * @file
 * Contains \Drupal\ces_import\ImportOffers.
 *
 */
namespace Drupal\ces_import;

use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;
use Drupal\smallads\Entity\SmallAdInterface;

class ImportOffers extends ImportBase {

  protected $entityTypeId = 'smallad';
  protected $bundle = 'offer';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ces_import_offers_form';
  }

    /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $form['#redirect'] = \Drupal\Core\Url::fromRoute('entity.smallad.collection');
    return $form;
  }

  public function deleteAll() {
    foreach ($this->entityTypeManager->getStorage('smallad')->loadByProperties(['type' => $this->bundle]) as $entity) {
      $entity->delete();
    }
  }

  function newTerms($rows) {
    $needed_term_names = [];
    foreach ($rows as $rownum => $row) {
      //check for new categories that will need creating
      if (array_key_exists('categories', $row) && strlen($row['categories'])) {
        $needed_term_names[] = trim($row['categories']);
      }
    }
    if ($needed_term_names) {
      $existing_terms = $this->entityTypeManager
        ->getStorage('taxonomy_term')
        ->loadByProperties(['name' => $needed_term_names, 'vid' => SMALLAD_CATEGORIES]);
      $existing_term_names = [];
      foreach ($existing_terms as $term) {
        $existing_term_names[] = $term->label();
      }
      $new_term_names = array_diff(array_unique($needed_term_names), $existing_term_names );
      if ($new_term_names) {
        foreach ($new_term_names as $name) {
          Term::Create(['name' => $name, 'vid' => SMALLAD_CATEGORIES])->save();
        }
        drupal_set_message('Created new terms '.implode(', ', $new_term_names));
      }
    }
  }

  function columns() {
    return [
      'ces_uid',//: user id e.g. abcd1234
      'title',//string
      'categories', //comma separated term names
      'body', //string
      'created', //item created date, format:  "2016-01-15 19:10:33"
      'changed', //last changed date, format:  "2016-01-15 19:10:33"
      'expires', //date, format:  "2016-01-15 19:10:33"
      'scope', //0 means just the owner, 2 means every authenticated user
      'picture'
    ];
  }

  function bodyProcess(SmallAdInterface $entity, $val, $allfields) {
    $entity->body->value = $val;
    $entity->body->format = 'basic';
  }

  function ces_uidProcess(SmallAdInterface $entity, $val, $allfields) {
    $owner = $this->entityTypeManager->getStorage('user')->loadByProperties(['name' => $val]);
    $entity->setOwner(reset($owner));
  }

  function pictureProcess(SmallAdInterface $entity, $src, $allfields) {
    if ($src) {
      //\Drupal::logger('ces')->debug($src);//this was causing an overload
      $info = pathinfo($src);
      $newpath = 'public://'.$info['basename'];
      if (copy($src, $newpath)) {
        $file = \Drupal\file\Entity\File::create(['uri' => $newpath]);
        $file->save();
        $entity->set('image', $file);
      }
    }
  }


  function getCsvRows($data) {
    $rows = parent::getCsvRows($data);
    foreach ($rows as $rownum => $row) {
      $name = trim($row['ces_uid']);
      if (!$this->entityTypeManager->getStorage('user')->loadByProperties(['name' => $name])) {
        //this is triggering for every ad
        drupal_set_message(
          t("@type seems to have unknown owner @owner.",
          [
            '@type'=> $this->bundle, 
            '@owner' => $name
          ]
        ),
        'warning'
      );
//        unset($rows[$rownum]);
      }
    }
    if ($new = $this->newTerms($rows)) {
      drupal_set_message(
        $this->t(
          'The following new terms have been created: @terms',
          ['@terms' => implode(', ', $new)]
        )
      );
    }
    return $rows;
  }

  //
  function categoriesProcess(SmallAdInterface $entity, $val, $allfields) {
    try {
      $terms = $this->entityTypeManager->getStorage('taxonomy_term')->loadByProperties(['name' => $val]);
      if (!$terms) {
        throw new \Exception('unable to load term with name '.$val);
      }
      $entity->categories->entity = reset($terms);
    }
    catch (\Exception $e) {
      drupal_set_message($e->getMessage());
    }
  }

  function makeBatch($rows, $delete = FALSE) {
    $batch = parent::makeBatch($rows, $delete);
    $batch['title'] = 'Importing Offers';
    return $batch;
  }


}