<?php

/**
 * @file
 * Contains \Drupal\ces_import\ImportUsers.
 *
 */
namespace Drupal\ces_import;

use Drupal\Core\Form\FormStateInterface;
use \Drupal\user\UserInterface;

class ImportUsers extends ImportBase {

  protected $entityTypeId = 'user';
  protected $bundle = 'user';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ces_import_users_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $form['#redirect'] = \Drupal\Core\Url::fromRoute('entity.user.collection');
    return $form;
  }

  public function deleteAll() {
    foreach ($this->entityTypeManager->getStorage('user')->loadMultiple() as $entity) {
      if ($entity->id() < 2) {
        continue;//don't delete user 1
      }
      $entity->delete();
    }
  }

  protected function createEntity() {
    $entity = parent::createEntity($this->bundle);
    $entity->activate();
    return $entity;
  }

  protected function columns() {
    return [
      'name',
      'mail',
      'password',
      'role',
      'firstname',
      'lastname',
      'address.organization',
      'created',
      'access',
      'address.line1',
      'address.line2',
      'address.dependent_locality',
      'address.administrative_area',
      //'address.postal_code', we have no way to force these to be valid
      'address.country_code',
      'address.locality',
      'phones',
      'phones2',
      'notes_admin',
      'privacy_phone1',
      'privacy_phone2',
      'privacy_address',
      'portrait'
    ];
  }

  function nameProcess(UserInterface $account, $val, $allfields) {
    $this->required($val);
    $account->setUsername($val);
  }

  function passwordProcess(UserInterface $account, $val, $allfields) {
    $account->setPassword($val);
  }

  function mailProcess(UserInterface $account, $val, $allfields) {
    if (empty($val) or !preg_match('/^.+\@\S+\.\S+$/', $val)) {
      $sitemail = \Drupal::config('system.site')->get('mail');
      $suffix = substr($sitemail, strpos($sitemail, '@'));
      if (!$suffix) {
        throw new Exception('Site mail cannot be used for creating dummy emails: '.$sitemail);
      }
      $val = $allfields['name'] . $suffix;
    }
    $account->setEmail($val);
  }

  /**
   *
   * //ind|fam|com|org|pub|adm
   */
  function roleProcess(UserInterface $account, $val, &$allfields) {
    if (strlen($allfields['firstname']) > 25 or strlen($allfields['lastname']) > 25) {
      $val = 'org';
      $allfields['address.organization'] = $allfields['firstname'].$allfields['lastname'];
      $allfields['firstname'] = substr($allfields['firstname'], 0, 25);
      $allfields['lastname'] = substr($allfields['lastname'], 0, 25);
    }
    $account->addRole('trader');
    switch($val) {
      case 'ind':
      case 'fam':
      case 'com':
      case 'org':
        $account->walletType = $val;
        break;
      case 'pub'://publicly visible wallet permission
        drupal_set_message('public/private wallet setting not yet supported');
        break;
      case 'adm':
        $account->addRole('committee');
        break;
      default:
        throw new Exception('Unknown role: '.$val);
    }
  }

  function coordinatorProcess(UserInterface $account, $val, &$allfields) {
    $account->addRole('superneighbour');
  }


  function dependent_localityProcess(UserInterface $account, $val, $allfields) {
    //temporary - a dependent locality will be a group later on and thus the address will reference it
    $this->required($val, 'Brisbane');
    $account->address->dependent_locality = $val;
  }

  function phones2Process(UserInterface $account, $val, $allfields) {
    $vals = [$allfields['phones'], $val];
    $account->set('phones', $vals);
  }


  function notesProcess(UserInterface $account, $val, $allfields) {
    $val = str_replace('<br />', '\n', $val);
    $account->set('notes', $val);
  }

  /**
   * TEMP
   */
  function portraitProcess(UserInterface $account, $src, $allfields) {
    if ($src) {
      //\Drupal::logger('ces')->debug($src);//this was causing an overload
      $info = pathinfo($src);
      $newpath = 'private://portraits/'.$info['basename'];
      if (copy($src, $newpath)) {
        $file = \Drupal\file\Entity\File::create(['uri' => $newpath]);
        $file->save();
        $account->set('portrait', $file);
      }
    }
  }

  //batch callback.
  //the sandbox seems to be empty every time, unfortunately
  function saveEntities($rows, &$sandbox) {
    foreach ($rows as $key => $row) {
      if (empty($row['password'])) {
        unset($row[$key]);
      }
    }
    parent::saveEntities($rows, $sandbox);
    foreach ($rows as $rownum => $row) {
      $val = [
        'phone1' => $row['privacy_phone1'],
        'phone2' => $row['privacy_phone2'],
        'address' => $row['privacy_address']
      ];
      $user_ids = $this->entityTypeManager
        ->getStorage('user')->getQuery()
        ->condition('name', $row['name'])
        ->execute();
      \Drupal::keyValue('user.privacy')
        ->set(reset($user_ids), $val);
    }
  }

  function makeBatch($rows, $delete = FALSE) {
    $batch = parent::makeBatch($rows, $delete);
    $batch['title'] = 'Importing users';
    return $batch;
  }

  function privacy_phone1Process() {}
  function privacy_phone2Process() {}
  function privacy_addressProcess() {}

}
